==> pluto_grid-4.0.1.tar.gz: lib/src/manager/state/dragging_row_state.dart <==
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:pluto_grid/pluto_grid.dart';

abstract class IDraggingRowState {
  bool get isDraggingRow;

  List<PlutoRow> get dragRows;

  int? get dragTargetRowIdx;

  bool get canRowDrag;

  void setIsDraggingRow(
    bool flag, {
    bool notify = true,
  });

  void setDragRows(
    List<PlutoRow> rows, {
    bool notify = true,
  });

  void setDragTargetRowIdx(
    int rowIdx, {
    bool notify = true,
  });

  bool isRowIdxDragTarget(int rowIdx);

  bool isRowIdxTopDragTarget(int rowIdx);

  bool isRowIdxBottomDragTarget(int rowIdx);

  bool isRowBeingDragged(Key rowKey);
}

mixin DraggingRowState implements IPlutoGridState {
  @override
  bool get isDraggingRow => _isDraggingRow;

  bool _isDraggingRow = false;

  @override
  List<PlutoRow> get dragRows => _dragRows;

  List<PlutoRow> _dragRows = [];

  @override
  int? get dragTargetRowIdx => _dragTargetRowIdx;

  int? _dragTargetRowIdx;

  @override
  bool get canRowDrag => !hasFilter && !hasSortedColumn;

  @override
  void setIsDraggingRow(
    bool flag, {
    bool notify = true,
  }) {
    if (_isDraggingRow == flag) {
      return;
    }

    _isDraggingRow = flag;

    _clearDraggingState();

    if (notify) {
      notifyListeners();
    }
  }

  @override
  void setDragRows(
    List<PlutoRow> rows, {
    bool notify = true,
  }) {
    _dragRows = rows;

    if (notify) {
      notifyListeners();
    }
  }

  @override
  void setDragTargetRowIdx(
    int? rowIdx, {
    bool notify = true,
  }) {
    if (_dragTargetRowIdx == rowIdx) {
      return;
    }

    _dragTargetRowIdx = rowIdx;

    if (notify) {
      notifyListeners();
    }
  }

  @override
  bool isRowIdxDragTarget(int? rowIdx) {
    return rowIdx != null &&
        _dragTargetRowIdx != null &&
        _dragTargetRowIdx! <= rowIdx &&
        rowIdx < _dragTargetRowIdx! + _dragRows.length;
  }

  @override
  bool isRowIdxTopDragTarget(int? rowIdx) {
    return rowIdx != null &&
        _dragTargetRowIdx != null &&
        _dragTargetRowIdx == rowIdx &&
        rowIdx + _dragRows.length <= refRows.length;
  }

  @override
  bool isRowIdxBottomDragTarget(int? rowIdx) {
    return rowIdx != null &&
        _dragTargetRowIdx != null &&
        rowIdx == _dragTargetRowIdx! + _dragRows.length - 1;
  }

  @override
  bool isRowBeingDragged(Key? rowKey) {
    return rowKey != null &&
        _isDraggingRow == true &&
        dragRows.firstWhereOrNull((element) => element.key == rowKey) != null;
  }

  void _clearDraggingState() {
    _dragRows = [];

    _dragTargetRowIdx = null;
  }
}

==> pluto_grid-7.0.2.tar.gz: lib/src/manager/state/dragging_row_state.dart <==
import 'package:collection/collection.dart';
import 'package:flutter/material.dart';
import 'package:pluto_grid/pluto_grid.dart';

abstract class IDraggingRowState {
  bool get isDraggingRow;

  List<PlutoRow> get dragRows;

  int? get dragTargetRowIdx;

  bool get canRowDrag;

  void setIsDraggingRow(
    bool flag, {
    bool notify = true,
  });

  void setDragRows(
    List<PlutoRow> rows, {
    bool notify = true,
  });

  void setDragTargetRowIdx(
    int rowIdx, {
    bool notify = true,
  });

  bool isRowIdxDragTarget(int rowIdx);

  bool isRowIdxTopDragTarget(int rowIdx);

  bool isRowIdxBottomDragTarget(int rowIdx);

  bool isRowBeingDragged(Key rowKey);
}

class _State {
  bool _isDraggingRow = false;

  List<PlutoRow> _dragRows = [];

  int? _dragTargetRowIdx;
}

mixin DraggingRowState implements IPlutoGridState {
  final _State _state = _State();

  @override
  bool get isDraggingRow => _state._isDraggingRow;

  @override
  List<PlutoRow> get dragRows => _state._dragRows;

  @override
  int? get dragTargetRowIdx => _state._dragTargetRowIdx;

  @override
  bool get canRowDrag => !hasFilter && !hasSortedColumn && !enabledRowGroups;

  @override
  void setIsDraggingRow(
    bool flag, {
    bool notify = true,
  }) {
    if (isDraggingRow == flag) {
      return;
    }

    _state._isDraggingRow = flag;

    _clearDraggingState();

    notifyListeners(notify, setIsDraggingRow.hashCode);
  }

  @override
  void setDragRows(
    List<PlutoRow> rows, {
    bool notify = true,
  }) {
    _state._dragRows = rows;

    notifyListeners(notify, setDragRows.hashCode);
  }

  @override
  void setDragTargetRowIdx(
    int? rowIdx, {
    bool notify = true,
  }) {
    if (dragTargetRowIdx == rowIdx) {
      return;
    }

    _state._dragTargetRowIdx = rowIdx;

    notifyListeners(notify, setDragTargetRowIdx.hashCode);
  }

  @override
  bool isRowIdxDragTarget(int? rowIdx) {
    return rowIdx != null &&
        dragTargetRowIdx != null &&
        dragTargetRowIdx! <= rowIdx &&
        rowIdx < dragTargetRowIdx! + dragRows.length;
  }

  @override
  bool isRowIdxTopDragTarget(int? rowIdx) {
    return rowIdx != null &&
        dragTargetRowIdx != null &&
        dragTargetRowIdx == rowIdx &&
        rowIdx + dragRows.length <= refRows.length;
  }

  @override
  bool isRowIdxBottomDragTarget(int? rowIdx) {
    return rowIdx != null &&
        dragTargetRowIdx != null &&
        rowIdx == dragTargetRowIdx! + dragRows.length - 1;
  }

  @override
  bool isRowBeingDragged(Key? rowKey) {
    return rowKey != null &&
        isDraggingRow == true &&
        dragRows.firstWhereOrNull((element) => element.key == rowKey) != null;
  }

  void _clearDraggingState() {
    _state._dragRows = [];

    _state._dragTargetRowIdx = null;
  }
}
